@@ -5,7 +5,7 @@ set -e
 shopt -s extglob
 
 if [ -z "$1" ] || [ -z "$2" ]; then
-  echo "Usage: $0 version destdir [--no-package] [--dev-build] [--winelib]"
+  echo "Usage: $0 version destdir [--no-package] [--dev-build]"
   exit 1
 fi
 
@@ -23,7 +23,7 @@ shift 2
 
 opt_nopackage=0
 opt_devbuild=0
-opt_winelib=0
+opt_buildid=false
 
 crossfile="build-win"
 
@@ -36,9 +36,8 @@ while [ $# -gt 0 ]; do
     opt_nopackage=1
     opt_devbuild=1
     ;;
-  "--winelib")
-    opt_winelib=1
-    crossfile="build-wine"
+  "--build-id")
+    opt_buildid=true
     ;;
   *)
     echo "Unrecognized option: $1" >&2
@@ -60,16 +59,15 @@ function build_arch {
         --bindir "x$1"                                \
         --libdir "x$1"                                \
         -Denable_tests=false                          \
+        -Dbuild_id=$opt_buildid                       \
         "$DXVK_BUILD_DIR/build.$1"
 
   cd "$DXVK_BUILD_DIR/build.$1"
   ninja install
 
   if [ $opt_devbuild -eq 0 ]; then
-    if [ $opt_winelib -eq 0 ]; then
-      # get rid of some useless .a files
-      rm "$DXVK_BUILD_DIR/x$1/"*.!(dll)
-    fi
+    # get rid of some useless .a files
+    rm "$DXVK_BUILD_DIR/x$1/"*.!(dll)
     rm -R "$DXVK_BUILD_DIR/build.$1"
   fi
 }
