@@ -2,6 +2,8 @@
 
 #include "../util/util_time.h"
 
+#include "../util/sync/sync_signal_win32.h"
+
 #include "d3d11_context.h"
 #include "d3d11_state_object.h"
 
@@ -19,10 +21,6 @@ namespace dxvk {
       const Rc<DxvkDevice>& Device);
     ~D3D11ImmediateContext();
     
-    ULONG STDMETHODCALLTYPE AddRef();
-    
-    ULONG STDMETHODCALLTYPE Release();
-    
     D3D11_DEVICE_CONTEXT_TYPE STDMETHODCALLTYPE GetType();
     
     UINT STDMETHODCALLTYPE GetContextFlags();
@@ -114,7 +112,8 @@ namespace dxvk {
     DxvkCsThread m_csThread;
     bool         m_csIsBusy = false;
 
-    std::atomic<uint32_t> m_refCount = { 0 };
+    Rc<sync::Win32Fence> m_eventSignal;
+    uint64_t             m_eventCount = 0;
 
     dxvk::high_resolution_clock::time_point m_lastFlush
       = dxvk::high_resolution_clock::now();
@@ -148,6 +147,8 @@ namespace dxvk {
     void EmitCsChunk(DxvkCsChunkRef&& chunk);
 
     void FlushImplicit(BOOL StrongHint);
+
+    void SignalEvent(HANDLE hEvent);
     
   };
   
