@@ -35,6 +35,13 @@ namespace dxvk {
     /// TGSM in compute shaders before reading it.
     bool zeroInitWorkgroupMemory;
 
+    /// Force thread-group shared memory barriers
+    ///
+    /// Workaround for compute shaders that read and
+    /// write from the same shared memory location
+    /// without explicit synchronization.
+    bool forceTgsmBarriers;
+
     /// Use relaxed memory barriers
     ///
     /// May improve performance in some games,
@@ -57,6 +64,9 @@ namespace dxvk {
     /// Declare vertex positions in shaders as invariant
     bool invariantPosition;
 
+    /// Enable float control bits
+    bool floatControls;
+
     /// Back buffer count for the Vulkan swap chain.
     /// Overrides DXGI_SWAP_CHAIN_DESC::BufferCount.
     int32_t numBackBuffers;
@@ -65,6 +75,10 @@ namespace dxvk {
     /// passed to IDXGISwapChain::Present.
     int32_t syncInterval;
 
+    /// Tear-free mode if vsync is disabled
+    /// Tearing mode if vsync is enabled
+    Tristate tearFree;
+
     /// Override maximum frame latency if the app specifies
     /// a higher value. May help with frame timing issues.
     int32_t maxFrameLatency;
