@@ -4,6 +4,8 @@
 
 #include "../dxvk/hud/dxvk_hud.h"
 
+#include "../dxvk/dxvk_swapchain_blitter.h"
+
 #include "../util/sync/sync_signal_win32.h"
 
 namespace dxvk {
@@ -11,17 +13,6 @@ namespace dxvk {
   class D3D11Device;
   class D3D11DXGIDevice;
 
-  /**
-   * \brief Gamma control point
-   * 
-   * Control points are stored as normalized
-   * 16-bit unsigned integer values that will
-   * be converted back to floats in the shader.
-   */
-  struct D3D11_VK_GAMMA_CP {
-    uint16_t R, G, B, A;
-  };
-
   class D3D11SwapChain : public ComObject<IDXGIVkSwapChain> {
     constexpr static uint32_t DefaultFrameLatency = 1;
   public:
@@ -87,42 +78,24 @@ namespace dxvk {
 
     Com<D3D11DXGIDevice, false> m_dxgiDevice;
     
-    D3D11Device*            m_parent;
-    HWND                    m_window;
+    D3D11Device*              m_parent;
+    HWND                      m_window;
 
-    DXGI_SWAP_CHAIN_DESC1   m_desc;
+    DXGI_SWAP_CHAIN_DESC1     m_desc;
 
-    Rc<DxvkDevice>          m_device;
-    Rc<DxvkContext>         m_context;
+    Rc<DxvkDevice>            m_device;
+    Rc<DxvkContext>           m_context;
 
-    Rc<vk::Presenter>       m_presenter;
+    Rc<vk::Presenter>         m_presenter;
 
-    Rc<DxvkShader>          m_vertShader;
-    Rc<DxvkShader>          m_fragShader;
+    Rc<DxvkImage>             m_swapImage;
+    Rc<DxvkImageView>         m_swapImageView;
+    Rc<DxvkSwapchainBlitter>  m_blitter;
 
-    Rc<DxvkSampler>         m_samplerFitting;
-    Rc<DxvkSampler>         m_samplerScaling;
+    Rc<hud::Hud>              m_hud;
 
-    Rc<DxvkSampler>         m_gammaSampler;
-    Rc<DxvkImage>           m_gammaTexture;
-    Rc<DxvkImageView>       m_gammaTextureView;
-
-    Rc<DxvkImage>           m_swapImage;
-    Rc<DxvkImage>           m_swapImageResolve;
-    Rc<DxvkImageView>       m_swapImageView;
-
-    Rc<hud::Hud>            m_hud;
-
-    DxvkInputAssemblyState  m_iaState;
-    DxvkRasterizerState     m_rsState;
-    DxvkMultisampleState    m_msState;
-    DxvkDepthStencilState   m_dsState;
-    DxvkLogicOpState        m_loState;
-    DxvkBlendMode           m_blendMode;
-
-    D3D11Texture2D*         m_backBuffer = nullptr;
-
-    DxvkSubmitStatus        m_presentStatus;
+    D3D11Texture2D*           m_backBuffer = nullptr;
+    DxvkSubmitStatus          m_presentStatus;
 
     std::vector<Rc<DxvkImageView>> m_imageViews;
 
@@ -155,21 +128,11 @@ namespace dxvk {
 
     void CreateBackBuffer();
 
-    void CreateGammaTexture(
-            UINT                NumControlPoints,
-      const D3D11_VK_GAMMA_CP*  pControlPoints);
-    
-    void DestroyFrameLatencyEvent();
+    void CreateBlitter();
 
-    void DestroyGammaTexture();
-    
     void CreateHud();
 
-    void InitRenderState();
-
-    void InitSamplers();
-
-    void InitShaders();
+    void DestroyFrameLatencyEvent();
 
     void SignalFrameLatencyEvent();
 
