@@ -62,7 +62,9 @@ namespace dxvk {
     D3D11CommonTexture(
             D3D11Device*                pDevice,
       const D3D11_COMMON_TEXTURE_DESC*  pDesc,
-            D3D11_RESOURCE_DIMENSION    Dimension);
+            D3D11_RESOURCE_DIMENSION    Dimension,
+            DXGI_USAGE                  DxgiUsage,
+            VkImage                     vkImage);
     
     ~D3D11CommonTexture();
     
@@ -77,6 +79,17 @@ namespace dxvk {
       return &m_desc;
     }
 
+    /**
+     * \brief Special DXGI usage flags
+     *
+     * Flags that are set in addition to the bind
+     * flags. Zero for application-created textures.
+     * \returns DXGI usage flags
+     */
+    DXGI_USAGE GetDxgiUsage() const {
+      return m_dxgiUsage;
+    }
+
     /**
      * \brief Counts number of subresources
      * \returns Number of subresources
@@ -172,12 +185,6 @@ namespace dxvk {
      */
     DXGI_VK_FORMAT_MODE GetFormatMode() const;
     
-    /**
-     * \brief Retrieves parent D3D11 device
-     * \param [out] ppDevice The device
-     */
-    void GetDevice(ID3D11Device** ppDevice) const;
-    
     /**
      * \brief Checks whether a view can be created for this textue
      * 
@@ -207,9 +214,10 @@ namespace dxvk {
     
   private:
     
-    Com<D3D11Device>              m_device;
+    D3D11Device* const            m_device;
     D3D11_COMMON_TEXTURE_DESC     m_desc;
     D3D11_COMMON_TEXTURE_MAP_MODE m_mapMode;
+    DXGI_USAGE                    m_dxgiUsage;
     
     Rc<DxvkImage>                 m_image;
     std::vector<Rc<DxvkBuffer>>   m_buffers;
@@ -380,9 +388,6 @@ namespace dxvk {
             REFIID  riid,
             void**  ppvObject) final;
     
-    void STDMETHODCALLTYPE GetDevice(
-            ID3D11Device **ppDevice) final;
-    
     void STDMETHODCALLTYPE GetType(
             D3D11_RESOURCE_DIMENSION *pResourceDimension) final;
     
@@ -421,6 +426,12 @@ namespace dxvk {
     D3D11Texture2D(
             D3D11Device*                pDevice,
       const D3D11_COMMON_TEXTURE_DESC*  pDesc);
+
+    D3D11Texture2D(
+            D3D11Device*                pDevice,
+      const D3D11_COMMON_TEXTURE_DESC*  pDesc,
+            DXGI_USAGE                  DxgiUsage,
+            VkImage                     vkImage);
     
     ~D3D11Texture2D();
     
@@ -428,9 +439,6 @@ namespace dxvk {
             REFIID  riid,
             void**  ppvObject) final;
     
-    void STDMETHODCALLTYPE GetDevice(
-            ID3D11Device **ppDevice) final;
-    
     void STDMETHODCALLTYPE GetType(
             D3D11_RESOURCE_DIMENSION *pResourceDimension) final;
     
@@ -479,9 +487,6 @@ namespace dxvk {
             REFIID  riid,
             void**  ppvObject) final;
     
-    void STDMETHODCALLTYPE GetDevice(
-            ID3D11Device **ppDevice) final;
-    
     void STDMETHODCALLTYPE GetType(
             D3D11_RESOURCE_DIMENSION *pResourceDimension) final;
     
