@@ -77,34 +77,6 @@ namespace dxvk {
   }
   
 
-  VkBufferUsageFlags GetBufferUsageFlags(UINT BindFlags) {
-    VkBufferUsageFlags usage = 0;
-
-    if (BindFlags & D3D11_BIND_SHADER_RESOURCE)
-      usage |= VK_BUFFER_USAGE_UNIFORM_TEXEL_BUFFER_BIT;
-    if (BindFlags & D3D11_BIND_UNORDERED_ACCESS)
-      usage |= VK_BUFFER_USAGE_STORAGE_TEXEL_BUFFER_BIT;
-    
-    return 0;
-  }
-  
-
-  VkImageUsageFlags GetImageUsageFlags(UINT BindFlags) {
-    VkImageUsageFlags usage = 0;
-
-    if (BindFlags & D3D11_BIND_DEPTH_STENCIL)
-      usage |= VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT;
-    if (BindFlags & D3D11_BIND_RENDER_TARGET)
-      usage |= VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT;
-    if (BindFlags & D3D11_BIND_SHADER_RESOURCE)
-      usage |= VK_IMAGE_USAGE_SAMPLED_BIT;
-    if (BindFlags & D3D11_BIND_UNORDERED_ACCESS)
-      usage |= VK_IMAGE_USAGE_STORAGE_BIT;
-    
-    return usage;
-  }
-
-
   VkFormatFeatureFlags GetBufferFormatFeatures(UINT BindFlags) {
     VkFormatFeatureFlags features = 0;
 
