@@ -34,12 +34,6 @@ namespace dxvk {
   VkShaderStageFlagBits GetShaderStage(
           DxbcProgramType           ProgramType);
   
-  VkBufferUsageFlags GetBufferUsageFlags(
-          UINT                      BindFlags);
-  
-  VkImageUsageFlags GetImageUsageFlags(
-          UINT                      BindFlags);
-  
   VkFormatFeatureFlags GetBufferFormatFeatures(
           UINT                      BindFlags);
 
