@@ -10,7 +10,8 @@ namespace dxvk {
           D3D11Device*                      pDevice,
           ID3D11Resource*                   pResource,
     const D3D11_SHADER_RESOURCE_VIEW_DESC1* pDesc)
-  : m_device(pDevice), m_resource(pResource), m_desc(*pDesc), m_d3d10(this) {
+  : D3D11DeviceChild<ID3D11ShaderResourceView1>(pDevice),
+    m_resource(pResource), m_desc(*pDesc), m_d3d10(this) {
     ResourceAddRefPrivate(m_resource);
 
     D3D11_COMMON_RESOURCE_DESC resourceDesc;
@@ -211,11 +212,6 @@ namespace dxvk {
   }
   
   
-  void STDMETHODCALLTYPE D3D11ShaderResourceView::GetDevice(ID3D11Device** ppDevice) {
-    *ppDevice = m_device.ref();
-  }
-  
-  
   void STDMETHODCALLTYPE D3D11ShaderResourceView::GetResource(ID3D11Resource** ppResource) {
     *ppResource = ref(m_resource);
   }
