@@ -68,8 +68,6 @@ namespace dxvk {
     DirtyFFPixelData,
     DirtyProgVertexShader,
     DirtySharedPixelShaderData,
-    UpDirtiedVertices,
-    UpDirtiedIndices,
     ValidSampleMask,
     DirtyDepthBounds,
     DirtyPointScale,
@@ -708,7 +706,7 @@ namespace dxvk {
             D3D9CommonTexture*      pResource,
             UINT                    Subresource);
 
-    void GenerateMips(
+    void EmitGenerateMips(
             D3D9CommonTexture* pResource);
 
     HRESULT LockBuffer(
@@ -728,22 +726,40 @@ namespace dxvk {
 
     int64_t DetermineInitialTextureMemory();
 
+    Rc<DxvkBuffer> CreateConstantBuffer(
+            bool                SSBO,
+            VkDeviceSize        Size,
+            DxsoProgramType     ShaderStage,
+            DxsoConstantBuffers BufferType);
+
     void CreateConstantBuffers();
 
     void SynchronizeCsThread();
 
     void Flush();
 
-    D3D9ShaderMasks GetShaderMasks();
-
     void UpdateActiveRTs(uint32_t index);
 
-    void UpdateActiveRTTextures(uint32_t index);
+    void UpdateActiveTextures(uint32_t index, DWORD combinedUsage);
 
-    void UpdateActiveHazards();
+    void UpdateActiveHazardsRT(uint32_t rtMask);
+
+    void UpdateActiveHazardsDS(uint32_t texMask);
 
     void MarkRenderHazards();
 
+    void UploadManagedTexture(D3D9CommonTexture* pResource);
+
+    void UploadManagedTextures(uint32_t mask);
+
+    void GenerateTextureMips(uint32_t mask);
+
+    void MarkTextureMipsDirty(D3D9CommonTexture* pResource);
+
+    void MarkTextureMipsUnDirty(D3D9CommonTexture* pResource);
+
+    void MarkTextureUploaded(D3D9CommonTexture* pResource);
+
     template <bool Points>
     void UpdatePointMode();
 
@@ -776,6 +792,10 @@ namespace dxvk {
       return m_state.renderStates[D3DRS_ZENABLE] && m_state.depthStencil != nullptr;
     }
 
+    inline bool IsClipPlaneEnabled() {
+      return m_state.renderStates[D3DRS_CLIPPLANEENABLE] != 0;
+    }
+
     void BindMultiSampleState();
     
     void BindBlendState();
@@ -827,7 +847,7 @@ namespace dxvk {
     
     uint32_t GetInstanceCount() const;
 
-    void PrepareDraw(D3DPRIMITIVETYPE PrimitiveType, bool up = false);
+    void PrepareDraw(D3DPRIMITIVETYPE PrimitiveType);
 
     template <DxsoProgramType ShaderStage>
     void BindShader(
@@ -982,7 +1002,7 @@ namespace dxvk {
     D3D9UPBufferSlice               m_upBuffer;
 
     const D3D9Options               m_d3d9Options;
-    const DxsoOptions               m_dxsoOptions;
+    DxsoOptions                     m_dxsoOptions;
 
     BOOL                            m_isSWVP;
 
@@ -992,9 +1012,7 @@ namespace dxvk {
 
     Com<D3D9Surface, false>         m_autoDepthStencil;
 
-    std::vector<
-      Com<D3D9SwapChainEx,
-      false>>                       m_swapchains;
+    Com<D3D9SwapChainEx, false>     m_implicitSwapchain;
 
     std::unordered_map<
       D3D9SamplerKey,
@@ -1004,7 +1022,8 @@ namespace dxvk {
 
     std::unordered_map<
       DWORD,
-      Com<D3D9VertexDecl>> m_fvfTable;
+      Com<D3D9VertexDecl,
+      false>>                       m_fvfTable;
 
     D3D9InputAssemblyState          m_iaState;
 
@@ -1021,8 +1040,22 @@ namespace dxvk {
 
     uint32_t                        m_activeRTs        = 0;
     uint32_t                        m_activeRTTextures = 0;
-    uint32_t                        m_activeHazards    = 0;
+    uint32_t                        m_activeDSTextures = 0;
+    uint32_t                        m_activeHazardsRT  = 0;
     uint32_t                        m_alphaSwizzleRTs  = 0;
+    uint32_t                        m_activeTextures   = 0;
+    uint32_t                        m_activeTexturesToUpload = 0;
+    uint32_t                        m_activeTexturesToGen    = 0;
+
+    uint32_t                        m_fetch4Enabled = 0;
+    uint32_t                        m_fetch4        = 0;
+    uint32_t                        m_lastFetch4    = 0;
+
+    uint32_t                        m_activeHazardsDS = 0;
+    uint32_t                        m_lastHazardsDS   = 0;
+
+    D3D9ShaderMasks                 m_vsShaderMasks = D3D9ShaderMasks();
+    D3D9ShaderMasks                 m_psShaderMasks = FixedFunctionMask;
 
     D3D9ViewportInfo                m_viewportInfo;
 
@@ -1042,8 +1075,6 @@ namespace dxvk {
 
     D3D9UPBufferSlice AllocUpBuffer(VkDeviceSize size);
 
-    D3D9SwapChainEx* GetInternalSwapchain(UINT index);
-
     bool ShouldRecord();
 
     HRESULT               CreateShaderModule(
@@ -1173,6 +1204,8 @@ namespace dxvk {
 
     void UpdateProjectionSpecConstant(uint32_t value);
 
+    void UpdateFetch4SpecConstant(uint32_t value);
+
   };
 
 }
