@@ -91,74 +91,83 @@ namespace dxvk {
           ? fogCtx.vFog
           : spvModule.opFAbs(floatType, z);
     }
+    uint32_t fogFactor;
+    if (!fogCtx.IsPixel && fogCtx.IsFixedFunction && fogCtx.IsPositionT) {
+      fogFactor = fogCtx.HasSpecular
+        ? spvModule.opCompositeExtract(floatType, fogCtx.Specular, 1, &wIndex)
+        : spvModule.constf32(1.0f);
+    } else {
+      uint32_t applyFogFactor = spvModule.allocateId();
 
-    uint32_t applyFogFactor = spvModule.allocateId();
+      std::array<SpirvPhiLabel, 4> fogVariables;
 
-    std::array<SpirvPhiLabel, 4> fogVariables;
+      std::array<SpirvSwitchCaseLabel, 4> fogCaseLabels = { {
+        { uint32_t(D3DFOG_NONE),      spvModule.allocateId() },
+        { uint32_t(D3DFOG_EXP),       spvModule.allocateId() },
+        { uint32_t(D3DFOG_EXP2),      spvModule.allocateId() },
+        { uint32_t(D3DFOG_LINEAR),    spvModule.allocateId() },
+      } };
 
-    std::array<SpirvSwitchCaseLabel, 4> fogCaseLabels = { {
-      { uint32_t(D3DFOG_NONE),      spvModule.allocateId() },
-      { uint32_t(D3DFOG_EXP),       spvModule.allocateId() },
-      { uint32_t(D3DFOG_EXP2),      spvModule.allocateId() },
-      { uint32_t(D3DFOG_LINEAR),    spvModule.allocateId() },
-    } };
+      spvModule.opSelectionMerge(applyFogFactor, spv::SelectionControlMaskNone);
+      spvModule.opSwitch(fogMode,
+        fogCaseLabels[D3DFOG_NONE].labelId,
+        fogCaseLabels.size(),
+        fogCaseLabels.data());
 
-
-    spvModule.opSelectionMerge(applyFogFactor, spv::SelectionControlMaskNone);
-    spvModule.opSwitch(fogMode,
-      fogCaseLabels[D3DFOG_NONE].labelId,
-      fogCaseLabels.size(),
-      fogCaseLabels.data());
-
-    for (uint32_t i = 0; i < fogCaseLabels.size(); i++) {
-      spvModule.opLabel(fogCaseLabels[i].labelId);
+      for (uint32_t i = 0; i < fogCaseLabels.size(); i++) {
+        spvModule.opLabel(fogCaseLabels[i].labelId);
         
-      fogVariables[i].labelId = fogCaseLabels[i].labelId;
-      fogVariables[i].varId   = [&] {
-        auto mode = D3DFOGMODE(fogCaseLabels[i].literal);
-        switch (mode) {
-          default:
-          // vFog
-          case D3DFOG_NONE: {
-            return fogCtx.IsPixel
-              ? fogCtx.vFog
-              : spvModule.constf32(1.0f);
+        fogVariables[i].labelId = fogCaseLabels[i].labelId;
+        fogVariables[i].varId   = [&] {
+          auto mode = D3DFOGMODE(fogCaseLabels[i].literal);
+          switch (mode) {
+            default:
+            // vFog
+            case D3DFOG_NONE: {
+              if (fogCtx.IsPixel)
+                return fogCtx.vFog;
+
+              if (fogCtx.IsFixedFunction && fogCtx.HasSpecular)
+                return spvModule.opCompositeExtract(floatType, fogCtx.Specular, 1, &wIndex);
+
+              return spvModule.constf32(1.0f);
+            }
+
+            // (end - d) / (end - start)
+            case D3DFOG_LINEAR: {
+              uint32_t fogFactor = spvModule.opFSub(floatType, fogEnd, depth);
+              fogFactor = spvModule.opFMul(floatType, fogFactor, fogScale);
+              fogFactor = spvModule.opNClamp(floatType, fogFactor, spvModule.constf32(0.0f), spvModule.constf32(1.0f));
+              return fogFactor;
+            }
+
+            // 1 / (e^[d * density])^2
+            case D3DFOG_EXP2:
+            // 1 / (e^[d * density])
+            case D3DFOG_EXP: {
+              uint32_t fogFactor = spvModule.opFMul(floatType, depth, fogDensity);
+
+              if (mode == D3DFOG_EXP2)
+                fogFactor = spvModule.opFMul(floatType, fogFactor, fogFactor);
+
+              // Provides the rcp.
+              fogFactor = spvModule.opFNegate(floatType, fogFactor);
+              fogFactor = spvModule.opExp(floatType, fogFactor);
+              return fogFactor;
+            }
           }
-
-          // (end - d) / (end - start)
-          case D3DFOG_LINEAR: {
-            uint32_t fogFactor = spvModule.opFSub(floatType, fogEnd, depth);
-            fogFactor = spvModule.opFMul(floatType, fogFactor, fogScale);
-            fogFactor = spvModule.opNClamp(floatType, fogFactor, spvModule.constf32(0.0f), spvModule.constf32(1.0f));
-            return fogFactor;
-          }
-
-          // 1 / (e^[d * density])^2
-          case D3DFOG_EXP2:
-          // 1 / (e^[d * density])
-          case D3DFOG_EXP: {
-            uint32_t fogFactor = spvModule.opFMul(floatType, depth, fogDensity);
-
-            if (mode == D3DFOG_EXP2)
-              fogFactor = spvModule.opFMul(floatType, fogFactor, fogFactor);
-
-            // Provides the rcp.
-            fogFactor = spvModule.opFNegate(floatType, fogFactor);
-            fogFactor = spvModule.opExp(floatType, fogFactor);
-            return fogFactor;
-          }
-        }
-      }();
+        }();
         
-      spvModule.opBranch(applyFogFactor);
+        spvModule.opBranch(applyFogFactor);
+      }
+
+      spvModule.opLabel(applyFogFactor);
+
+      fogFactor = spvModule.opPhi(floatType,
+        fogVariables.size(),
+        fogVariables.data());
     }
 
-    spvModule.opLabel(applyFogFactor);
-
-    uint32_t fogFactor = spvModule.opPhi(floatType,
-      fogVariables.size(),
-      fogVariables.data());
-
     uint32_t fogRetValue = 0;
 
     // Return the new color if we are doing this in PS
@@ -190,7 +199,7 @@ namespace dxvk {
   }
 
 
-  uint32_t SetupRenderStateBlock(SpirvModule& spvModule) {
+  uint32_t SetupRenderStateBlock(SpirvModule& spvModule, uint32_t count) {
     uint32_t floatType = spvModule.defFloatType(32);
     uint32_t vec3Type  = spvModule.defVectorType(floatType, 3);
 
@@ -208,44 +217,44 @@ namespace dxvk {
       floatType,
       floatType,
     }};
-    
-    uint32_t rsStruct = spvModule.defStructTypeUnique(rsMembers.size(), rsMembers.data());
+
+    uint32_t rsStruct = spvModule.defStructTypeUnique(count, rsMembers.data());
     uint32_t rsBlock = spvModule.newVar(
       spvModule.defPointerType(rsStruct, spv::StorageClassPushConstant),
       spv::StorageClassPushConstant);
     
+    spvModule.setDebugName         (rsBlock, "render_state");
+
     spvModule.setDebugName         (rsStruct, "render_state_t");
     spvModule.decorate             (rsStruct, spv::DecorationBlock);
-    spvModule.setDebugMemberName   (rsStruct, 0, "fog_color");
-    spvModule.memberDecorateOffset (rsStruct, 0, offsetof(D3D9RenderStateInfo, fogColor));
-    spvModule.setDebugMemberName   (rsStruct, 1, "fog_scale");
-    spvModule.memberDecorateOffset (rsStruct, 1, offsetof(D3D9RenderStateInfo, fogScale));
-    spvModule.setDebugMemberName   (rsStruct, 2, "fog_end");
-    spvModule.memberDecorateOffset (rsStruct, 2, offsetof(D3D9RenderStateInfo, fogEnd));
-    spvModule.setDebugMemberName   (rsStruct, 3, "fog_density");
-    spvModule.memberDecorateOffset (rsStruct, 3, offsetof(D3D9RenderStateInfo, fogDensity));
-    spvModule.setDebugMemberName   (rsStruct, 4, "alpha_ref");
-    spvModule.memberDecorateOffset (rsStruct, 4, offsetof(D3D9RenderStateInfo, alphaRef));
-    spvModule.setDebugMemberName   (rsStruct, 5, "point_size");
-    spvModule.memberDecorateOffset (rsStruct, 5, offsetof(D3D9RenderStateInfo, pointSize));
-    spvModule.setDebugMemberName   (rsStruct, 6, "point_size_min");
-    spvModule.memberDecorateOffset (rsStruct, 6, offsetof(D3D9RenderStateInfo, pointSizeMin));
-    spvModule.setDebugMemberName   (rsStruct, 7, "point_size_max");
-    spvModule.memberDecorateOffset (rsStruct, 7, offsetof(D3D9RenderStateInfo, pointSizeMax));
-    spvModule.setDebugMemberName   (rsStruct, 8, "point_scale_a");
-    spvModule.memberDecorateOffset (rsStruct, 8, offsetof(D3D9RenderStateInfo, pointScaleA));
-    spvModule.setDebugMemberName   (rsStruct, 9, "point_scale_b");
-    spvModule.memberDecorateOffset (rsStruct, 9, offsetof(D3D9RenderStateInfo, pointScaleB));
-    spvModule.setDebugMemberName   (rsStruct, 10, "point_scale_c");
-    spvModule.memberDecorateOffset (rsStruct, 10, offsetof(D3D9RenderStateInfo, pointScaleC));
-    
-    spvModule.setDebugName         (rsBlock, "render_state");
+
+    uint32_t memberIdx = 0;
+    auto SetMemberName = [&](const char* name, uint32_t offset) {
+      if (memberIdx >= count)
+        return;
+
+      spvModule.setDebugMemberName   (rsStruct, memberIdx, name);
+      spvModule.memberDecorateOffset (rsStruct, memberIdx, offset);
+      memberIdx++;
+    };
+
+    SetMemberName("fog_color",      offsetof(D3D9RenderStateInfo, fogColor));
+    SetMemberName("fog_scale",      offsetof(D3D9RenderStateInfo, fogScale));
+    SetMemberName("fog_end",        offsetof(D3D9RenderStateInfo, fogEnd));
+    SetMemberName("fog_density",    offsetof(D3D9RenderStateInfo, fogDensity));
+    SetMemberName("alpha_ref",      offsetof(D3D9RenderStateInfo, alphaRef));
+    SetMemberName("point_size",     offsetof(D3D9RenderStateInfo, pointSize));
+    SetMemberName("point_size_min", offsetof(D3D9RenderStateInfo, pointSizeMin));
+    SetMemberName("point_size_max", offsetof(D3D9RenderStateInfo, pointSizeMax));
+    SetMemberName("point_scale_a",  offsetof(D3D9RenderStateInfo, pointScaleA));
+    SetMemberName("point_scale_b",  offsetof(D3D9RenderStateInfo, pointScaleB));
+    SetMemberName("point_scale_c",  offsetof(D3D9RenderStateInfo, pointScaleC));
 
     return rsBlock;
   }
 
 
-  D3D9PointSizeInfoVS GetPointSizeInfoVS(SpirvModule& spvModule, uint32_t vPos, uint32_t vtx, uint32_t perVertPointSize, uint32_t rsBlock) {
+  D3D9PointSizeInfoVS GetPointSizeInfoVS(SpirvModule& spvModule, uint32_t vPos, uint32_t vtx, uint32_t perVertPointSize, uint32_t rsBlock, bool isFixedFunction) {
     uint32_t floatType  = spvModule.defFloatType(32);
     uint32_t floatPtr   = spvModule.defPointerType(floatType, spv::StorageClassPushConstant);
     uint32_t vec3Type   = spvModule.defVectorType(floatType, 3);
@@ -258,43 +267,44 @@ namespace dxvk {
       return spvModule.opLoad(floatType, spvModule.opAccessChain(floatPtr, rsBlock, 1, &index));
     };
 
-    uint32_t pointMode = spvModule.specConst32(uint32Type, 0);
-    spvModule.setDebugName(pointMode, "point_mode");
-    spvModule.decorateSpecId(pointMode, getSpecId(D3D9SpecConstantId::PointMode));
+    uint32_t value = perVertPointSize != 0 ? perVertPointSize : LoadFloat(D3D9RenderStateItem::PointSize);
 
-    uint32_t scaleBit  = spvModule.opBitFieldUExtract(uint32Type, pointMode, spvModule.consti32(0), spvModule.consti32(1));
-    uint32_t isScale   = spvModule.opIEqual(boolType, scaleBit, spvModule.constu32(1));
+    if (isFixedFunction) {
+      uint32_t pointMode = spvModule.specConst32(uint32Type, 0);
+      spvModule.setDebugName(pointMode, "point_mode");
+      spvModule.decorateSpecId(pointMode, getSpecId(D3D9SpecConstantId::PointMode));
 
-    uint32_t regularValue = perVertPointSize != 0 ? perVertPointSize : LoadFloat(D3D9RenderStateItem::PointSize);
+      uint32_t scaleBit  = spvModule.opBitFieldUExtract(uint32Type, pointMode, spvModule.consti32(0), spvModule.consti32(1));
+      uint32_t isScale   = spvModule.opIEqual(boolType, scaleBit, spvModule.constu32(1));
 
-    uint32_t scaleC = LoadFloat(D3D9RenderStateItem::PointScaleC);
-    uint32_t scaleB = LoadFloat(D3D9RenderStateItem::PointScaleB);
-    uint32_t scaleA = LoadFloat(D3D9RenderStateItem::PointScaleA);
+      uint32_t scaleC = LoadFloat(D3D9RenderStateItem::PointScaleC);
+      uint32_t scaleB = LoadFloat(D3D9RenderStateItem::PointScaleB);
+      uint32_t scaleA = LoadFloat(D3D9RenderStateItem::PointScaleA);
 
-    std::array<uint32_t, 4> indices = { 0, 1, 2, 3 };
+      std::array<uint32_t, 4> indices = { 0, 1, 2, 3 };
 
-    uint32_t vtx3;
-    if (vPos != 0) {
-      vPos = spvModule.opLoad(vec4Type, vPos);
+      uint32_t vtx3;
+      if (vPos != 0) {
+        vPos = spvModule.opLoad(vec4Type, vPos);
 
-      uint32_t rhw            = spvModule.opCompositeExtract(floatType, vPos, 1, &indices[3]);
-               rhw            = spvModule.opFDiv(floatType, spvModule.constf32(1.0f), rhw);
-      uint32_t pos3           = spvModule.opVectorShuffle(vec3Type, vPos, vPos, 3, indices.data());
-               vtx3           = spvModule.opVectorTimesScalar(vec3Type, pos3, rhw);
+        uint32_t rhw  = spvModule.opCompositeExtract(floatType, vPos, 1, &indices[3]);
+                 rhw  = spvModule.opFDiv(floatType, spvModule.constf32(1.0f), rhw);
+        uint32_t pos3 = spvModule.opVectorShuffle(vec3Type, vPos, vPos, 3, indices.data());
+                 vtx3 = spvModule.opVectorTimesScalar(vec3Type, pos3, rhw);
+      } else {
+                 vtx3 = spvModule.opVectorShuffle(vec3Type, vtx, vtx, 3, indices.data());
+      }
+
+      uint32_t DeSqr      = spvModule.opDot (floatType, vtx3, vtx3);
+      uint32_t De         = spvModule.opSqrt(floatType, DeSqr);
+      uint32_t scaleValue = spvModule.opFMul(floatType, scaleC, DeSqr);
+               scaleValue = spvModule.opFFma(floatType, scaleB, De, scaleValue);
+               scaleValue = spvModule.opFAdd(floatType, scaleA, scaleValue);
+               scaleValue = spvModule.opSqrt(floatType, scaleValue);
+               scaleValue = spvModule.opFDiv(floatType, value, scaleValue);
+
+      value = spvModule.opSelect(floatType, isScale, scaleValue, value);
     }
-    else {
-               vtx3           = spvModule.opVectorShuffle(vec3Type, vtx, vtx, 3, indices.data());
-    }
-
-    uint32_t DeSqr          = spvModule.opDot (floatType, vtx3, vtx3);
-    uint32_t De             = spvModule.opSqrt(floatType, DeSqr);
-    uint32_t scaleValue     = spvModule.opFMul(floatType, scaleC, DeSqr);
-             scaleValue     = spvModule.opFFma(floatType, scaleB, De, scaleValue);
-             scaleValue     = spvModule.opFAdd(floatType, scaleA, scaleValue);
-             scaleValue     = spvModule.opSqrt(floatType, scaleValue);
-             scaleValue     = spvModule.opFDiv(floatType, regularValue, scaleValue);
-
-    uint32_t value = spvModule.opSelect(floatType, isScale, scaleValue, regularValue);
 
     uint32_t min   = LoadFloat(D3D9RenderStateItem::PointSizeMin);
     uint32_t max   = LoadFloat(D3D9RenderStateItem::PointSizeMax);
@@ -418,6 +428,7 @@ namespace dxvk {
   enum class D3D9FFVSMembers {
     WorldViewMatrix,
     NormalMatrix,
+    InverseViewMatrix,
     ProjMatrix,
       
     Texcoord0,
@@ -462,6 +473,7 @@ namespace dxvk {
     struct {
       uint32_t worldview;
       uint32_t normal;
+      uint32_t inverseView;
       uint32_t proj;
 
       uint32_t texcoord[8];
@@ -580,6 +592,8 @@ namespace dxvk {
 
     void emitPsSharedConstants();
 
+    void emitVsClipping(uint32_t vtx);
+
     void alphaTestPS();
 
     bool isVS() { return m_programType == DxsoProgramType::VertexShader; }
@@ -607,6 +621,7 @@ namespace dxvk {
     uint32_t              m_uint32Type;
     uint32_t              m_vec4Type;
     uint32_t              m_vec3Type;
+    uint32_t              m_vec2Type;
     uint32_t              m_mat3Type;
     uint32_t              m_mat4Type;
 
@@ -647,6 +662,7 @@ namespace dxvk {
     m_uint32Type = m_module.defIntType(32, 0);
     m_vec4Type   = m_module.defVectorType(m_floatType, 4);
     m_vec3Type   = m_module.defVectorType(m_floatType, 3);
+    m_vec2Type   = m_module.defVectorType(m_floatType, 2);
     m_mat3Type   = m_module.defMatrixType(m_vec3Type, 3);
     m_mat4Type   = m_module.defMatrixType(m_vec4Type, 4);
 
@@ -1150,26 +1166,37 @@ namespace dxvk {
     fogCtx.HasFogInput = m_vsKey.Data.Contents.HasFog;
     fogCtx.vFog        = m_vs.in.FOG;
     fogCtx.oColor      = 0;
+    fogCtx.IsFixedFunction = true;
+    fogCtx.IsPositionT = m_vsKey.Data.Contents.HasPositionT;
+    fogCtx.HasSpecular = m_vsKey.Data.Contents.HasColor1;
+    fogCtx.Specular    = m_vs.in.COLOR[1];
     m_module.opStore(m_vs.out.FOG, DoFixedFunctionFog(m_module, fogCtx));
 
-    auto pointInfo = GetPointSizeInfoVS(m_module, 0, vtx, m_vs.in.POINTSIZE, m_rsBlock);
+    auto pointInfo = GetPointSizeInfoVS(m_module, 0, vtx, m_vs.in.POINTSIZE, m_rsBlock, true);
 
     uint32_t pointSize = m_module.opFClamp(m_floatType, pointInfo.defaultValue, pointInfo.min, pointInfo.max);
     m_module.opStore(m_vs.out.POINTSIZE, pointSize);
+
+    if (m_vsKey.Data.Contents.VertexClipping)
+      emitVsClipping(vtx);
   }
 
 
   void D3D9FFShaderCompiler::setupRenderStateInfo() {
-    m_rsBlock = SetupRenderStateBlock(m_module);
+    uint32_t count;
 
     if (m_programType == DxsoProgramType::PixelShader) {
       m_interfaceSlots.pushConstOffset = 0;
       m_interfaceSlots.pushConstSize   = offsetof(D3D9RenderStateInfo, pointSize);
+      count = 5;
     }
     else {
       m_interfaceSlots.pushConstOffset = offsetof(D3D9RenderStateInfo, pointSize);
       m_interfaceSlots.pushConstSize   = sizeof(float) * 6;
+      count = 11;
     }
+
+    m_rsBlock = SetupRenderStateBlock(m_module, count);
   }
 
 
@@ -1236,6 +1263,7 @@ namespace dxvk {
     std::array<uint32_t, uint32_t(D3D9FFVSMembers::MemberCount)> members = {
       m_mat4Type, // World
       m_mat4Type, // View
+      m_mat4Type, // InverseView
       m_mat4Type, // Proj
 
       m_mat4Type, // Texture0
@@ -1309,6 +1337,7 @@ namespace dxvk {
     uint32_t member = 0;
     m_module.setDebugMemberName(structType, member++, "WorldView");
     m_module.setDebugMemberName(structType, member++, "Normal");
+    m_module.setDebugMemberName(structType, member++, "InverseView");
     m_module.setDebugMemberName(structType, member++, "Projection");
 
     m_module.setDebugMemberName(structType, member++, "TexcoordTransform0");
@@ -1427,6 +1456,7 @@ namespace dxvk {
 
     m_vs.constants.worldview = LoadConstant(m_mat4Type, uint32_t(D3D9FFVSMembers::WorldViewMatrix));
     m_vs.constants.normal    = LoadConstant(m_mat4Type, uint32_t(D3D9FFVSMembers::NormalMatrix));
+    m_vs.constants.inverseView = LoadConstant(m_mat4Type, uint32_t(D3D9FFVSMembers::InverseViewMatrix));
     m_vs.constants.proj      = LoadConstant(m_mat4Type, uint32_t(D3D9FFVSMembers::ProjMatrix));
 
     for (uint32_t i = 0; i < caps::TextureStageCount; i++)
@@ -1474,7 +1504,7 @@ namespace dxvk {
       m_isgn.elemCount++;
     }
 
-    if (m_vsKey.Data.Contents.HasPointSize)
+    if (m_vsKey.Data.Contents.HasFog)
       m_vs.in.FOG = declareIO(true, DxsoSemantic{ DxsoUsage::Fog,   0 });
     else
       m_isgn.elemCount++;
@@ -1530,6 +1560,31 @@ namespace dxvk {
 
       bool processedTexture = false;
 
+      auto DoBumpmapCoords = [&](uint32_t baseCoords) {
+        uint32_t stage = i - 1;
+
+        uint32_t coords = baseCoords;
+        for (uint32_t i = 0; i < 2; i++) {
+          std::array<uint32_t, 4> indices = { 0, 1, 2, 3 };
+
+          uint32_t tc_m_n = m_module.opCompositeExtract(m_floatType, coords, 1, &i);
+
+          uint32_t offset = m_module.constu32(D3D9SharedPSStages_Count * stage + D3D9SharedPSStages_BumpEnvMat0 + i);
+          uint32_t bm     = m_module.opAccessChain(m_module.defPointerType(m_vec2Type, spv::StorageClassUniform),
+                                                   m_ps.sharedState, 1, &offset);
+                   bm     = m_module.opLoad(m_vec2Type, bm);
+
+          uint32_t t      = m_module.opVectorShuffle(m_vec2Type, texture, texture, 2, indices.data());
+
+          uint32_t dot    = m_module.opDot(m_floatType, bm, t);
+
+          uint32_t result = m_module.opFAdd(m_floatType, tc_m_n, dot);
+          coords  = m_module.opCompositeInsert(m_vec4Type, result, coords, 1, &i);
+        }
+
+        return coords;
+      };
+
       auto GetTexture = [&]() {
         if (!processedTexture) {
           SpirvImageOperands imageOperands;
@@ -1555,17 +1610,54 @@ namespace dxvk {
           else
             projIdx--;
 
+          uint32_t projValue = 0;
+
           if (m_fsKey.Stages[i].Contents.Projected) {
-            uint32_t projValue = m_module.opCompositeExtract(m_floatType, m_ps.in.TEXCOORD[i], 1, &projIdx);
+            projValue = m_module.opCompositeExtract(m_floatType, m_ps.in.TEXCOORD[i], 1, &projIdx);
             uint32_t insertIdx = texcoordCnt - 1;
             texcoord = m_module.opCompositeInsert(texcoord_t, projValue, texcoord, 1, &insertIdx);
           }
 
-          if (m_fsKey.Stages[i].Contents.Projected)
+          bool shouldProject = m_fsKey.Stages[i].Contents.Projected;
+
+          if (i != 0 && (
+            m_fsKey.Stages[i - 1].Contents.ColorOp == D3DTOP_BUMPENVMAP ||
+            m_fsKey.Stages[i - 1].Contents.ColorOp == D3DTOP_BUMPENVMAPLUMINANCE)) {
+            if (shouldProject) {
+              uint32_t projRcp = m_module.opFDiv(m_floatType, m_module.constf32(1.0), projValue);
+              texcoord = m_module.opVectorTimesScalar(m_vec4Type, texcoord, projRcp);
+            }
+
+            texcoord = DoBumpmapCoords(texcoord);
+
+            shouldProject = false;
+          }
+
+          if (shouldProject)
             texture = m_module.opImageSampleProjImplicitLod(m_vec4Type, imageVarId, texcoord, imageOperands);
           else
             texture = m_module.opImageSampleImplicitLod(m_vec4Type, imageVarId, texcoord, imageOperands);
 
+          if (i != 0 && m_fsKey.Stages[i - 1].Contents.ColorOp == D3DTOP_BUMPENVMAPLUMINANCE) {
+            uint32_t index = m_module.constu32(D3D9SharedPSStages_Count * (i - 1) + D3D9SharedPSStages_BumpEnvLScale);
+            uint32_t lScale = m_module.opAccessChain(m_module.defPointerType(m_floatType, spv::StorageClassUniform),
+                                                     m_ps.sharedState, 1, &index);
+                     lScale = m_module.opLoad(m_floatType, lScale);
+
+                     index = m_module.constu32(D3D9SharedPSStages_Count * (i - 1) + D3D9SharedPSStages_BumpEnvLOffset);
+            uint32_t lOffset = m_module.opAccessChain(m_module.defPointerType(m_floatType, spv::StorageClassUniform),
+                                                     m_ps.sharedState, 1, &index);
+                     lOffset = m_module.opLoad(m_floatType, lOffset);
+            
+            uint32_t zIndex = 2;
+            uint32_t scale = m_module.opCompositeExtract(m_floatType, texture, 1, &zIndex);
+                     scale = m_module.opFMul(m_floatType, scale, lScale);
+                     scale = m_module.opFAdd(m_floatType, scale, lOffset);
+                     scale = m_module.opFClamp(m_floatType, scale, m_module.constf32(0.0f), m_module.constf32(1.0));
+
+            texture = m_module.opVectorTimesScalar(m_vec4Type, texture, scale);
+          }
+
           uint32_t bool_t = m_module.defBoolType();
           uint32_t bvec4_t = m_module.defVectorType(bool_t, 4);
           std::array<uint32_t, 4> boundIndices = { m_ps.samplers[i].bound, m_ps.samplers[i].bound, m_ps.samplers[i].bound, m_ps.samplers[i].bound };
@@ -1749,12 +1841,10 @@ namespace dxvk {
             dst = Saturate(dst);
             break;
 
-          case D3DTOP_BUMPENVMAP:
-            Logger::warn("D3DTOP_BUMPENVMAP: not implemented");
-            break;
-
           case D3DTOP_BUMPENVMAPLUMINANCE:
-            Logger::warn("D3DTOP_BUMPENVMAPLUMINANCE: not implemented");
+          case D3DTOP_BUMPENVMAP:
+            // Load texture for the next stage...
+            texture = GetTexture();
             break;
 
           case D3DTOP_DOTPRODUCT3: {
@@ -1870,6 +1960,10 @@ namespace dxvk {
     fogCtx.vPos        = m_ps.in.POS;
     fogCtx.vFog        = m_ps.in.FOG;
     fogCtx.oColor      = current;
+    fogCtx.IsFixedFunction = true;
+    fogCtx.IsPositionT = false;
+    fogCtx.HasSpecular = false;
+    fogCtx.Specular    = 0;
     current = DoFixedFunctionFog(m_module, fogCtx);
 
     m_module.opStore(m_ps.out.COLOR, current);
@@ -2039,6 +2133,77 @@ namespace dxvk {
   }
 
 
+  void D3D9FFShaderCompiler::emitVsClipping(uint32_t vtx) {
+    uint32_t worldPos = m_module.opMatrixTimesVector(m_vec4Type, m_vs.constants.inverseView, vtx);
+
+    uint32_t clipPlaneCountId = m_module.constu32(caps::MaxClipPlanes);
+    
+    uint32_t floatType = m_module.defFloatType(32);
+    uint32_t vec4Type  = m_module.defVectorType(floatType, 4);
+    
+    // Declare uniform buffer containing clip planes
+    uint32_t clipPlaneArray  = m_module.defArrayTypeUnique(vec4Type, clipPlaneCountId);
+    uint32_t clipPlaneStruct = m_module.defStructTypeUnique(1, &clipPlaneArray);
+    uint32_t clipPlaneBlock  = m_module.newVar(
+      m_module.defPointerType(clipPlaneStruct, spv::StorageClassUniform),
+      spv::StorageClassUniform);
+    
+    m_module.decorateArrayStride  (clipPlaneArray, 16);
+    
+    m_module.setDebugName         (clipPlaneStruct, "clip_info_t");
+    m_module.setDebugMemberName   (clipPlaneStruct, 0, "clip_planes");
+    m_module.decorate             (clipPlaneStruct, spv::DecorationBlock);
+    m_module.memberDecorateOffset (clipPlaneStruct, 0, 0);
+    
+    uint32_t bindingId = computeResourceSlotId(
+      DxsoProgramType::VertexShader,
+      DxsoBindingType::ConstantBuffer,
+      DxsoConstantBuffers::VSClipPlanes);
+    
+    m_module.setDebugName         (clipPlaneBlock, "clip_info");
+    m_module.decorateDescriptorSet(clipPlaneBlock, 0);
+    m_module.decorateBinding      (clipPlaneBlock, bindingId);
+    
+    DxvkResourceSlot resource;
+    resource.slot   = bindingId;
+    resource.type   = VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER;
+    resource.view   = VK_IMAGE_VIEW_TYPE_MAX_ENUM;
+    resource.access = VK_ACCESS_UNIFORM_READ_BIT;
+    m_resourceSlots.push_back(resource);
+    
+    // Declare output array for clip distances
+    uint32_t clipDistArray = m_module.newVar(
+      m_module.defPointerType(
+        m_module.defArrayType(floatType, clipPlaneCountId),
+        spv::StorageClassOutput),
+      spv::StorageClassOutput);
+
+    m_module.decorateBuiltIn(clipDistArray, spv::BuiltInClipDistance);
+    m_entryPointInterfaces.push_back(clipDistArray);
+
+    // Compute clip distances
+    for (uint32_t i = 0; i < caps::MaxClipPlanes; i++) {
+      std::array<uint32_t, 2> blockMembers = {{
+        m_module.constu32(0),
+        m_module.constu32(i),
+      }};
+      
+      uint32_t planeId = m_module.opLoad(vec4Type,
+        m_module.opAccessChain(
+          m_module.defPointerType(vec4Type, spv::StorageClassUniform),
+          clipPlaneBlock, blockMembers.size(), blockMembers.data()));
+      
+      uint32_t distId = m_module.opDot(floatType, worldPos, planeId);
+      
+      m_module.opStore(
+        m_module.opAccessChain(
+          m_module.defPointerType(floatType, spv::StorageClassOutput),
+          clipDistArray, 1, &blockMembers[1]),
+        distId);
+    }
+  }
+
+
   void D3D9FFShaderCompiler::alphaTestPS() {
     // Alpha testing
     uint32_t boolType = m_module.defBoolType();
@@ -2197,7 +2362,7 @@ namespace dxvk {
 
     if (dumpPath.size() != 0) {
       std::ofstream dumpStream(
-        str::format(dumpPath, "/", Name, ".spv"),
+        str::tows(str::format(dumpPath, "/", Name, ".spv").c_str()).c_str(),
         std::ios_base::binary | std::ios_base::trunc);
       
       m_shader->dump(dumpStream);
@@ -2317,4 +2482,4 @@ namespace dxvk {
 
   DxsoIsgn g_ffIsgn = CreateFixedFunctionIsgn();
 
-}
\ No newline at end of file
+}
