@@ -40,6 +40,7 @@ namespace dxvk {
     invariantPosition        = options.invariantPosition;
     enableRtOutputNanFixup   = options.enableRtOutputNanFixup;
     zeroInitWorkgroupMemory  = options.zeroInitWorkgroupMemory;
+    forceTgsmBarriers        = options.forceTgsmBarriers;
     dynamicIndexedConstantBufferAsSsbo = options.constantBufferRangeCheck;
 
     // Disable early discard on RADV (with LLVM) due to GPU hangs
@@ -56,6 +57,25 @@ namespace dxvk {
     
     // Apply shader-related options
     applyTristate(useSubgroupOpsForEarlyDiscard, device->config().useEarlyDiscard);
+
+    // Figure out float control flags to match D3D11 rules
+    if (options.floatControls) {
+      if (devInfo.khrShaderFloatControls.shaderSignedZeroInfNanPreserveFloat32)
+        floatControl.set(DxbcFloatControlFlag::PreserveNan32);
+      if (devInfo.khrShaderFloatControls.shaderSignedZeroInfNanPreserveFloat64)
+        floatControl.set(DxbcFloatControlFlag::PreserveNan64);
+
+      if (devInfo.khrShaderFloatControls.denormBehaviorIndependence != VK_SHADER_FLOAT_CONTROLS_INDEPENDENCE_NONE) {
+        if (devInfo.khrShaderFloatControls.shaderDenormFlushToZeroFloat32)
+          floatControl.set(DxbcFloatControlFlag::DenormFlushToZero32);
+        if (devInfo.khrShaderFloatControls.shaderDenormPreserveFloat64)
+          floatControl.set(DxbcFloatControlFlag::DenormPreserve64);
+      }
+    }
+
+    if (!devInfo.khrShaderFloatControls.shaderSignedZeroInfNanPreserveFloat32
+     || adapter->matchesDriver(DxvkGpuVendor::Amd, VK_DRIVER_ID_MESA_RADV_KHR, 0, VK_MAKE_VERSION(20, 3, 0)))
+      enableRtOutputNanFixup = true;
   }
   
 }
\ No newline at end of file
