@@ -66,7 +66,14 @@ namespace dxvk {
   
   
   DxgiAdapter::~DxgiAdapter() {
-    
+    if (m_eventThread.joinable()) {
+      std::unique_lock<std::mutex> lock(m_mutex);
+      m_eventCookie = ~0u;
+      m_cond.notify_one();
+
+      lock.unlock();
+      m_eventThread.join();
+    }
   }
   
   
@@ -82,6 +89,7 @@ namespace dxvk {
      || riid == __uuidof(IDXGIAdapter1)
      || riid == __uuidof(IDXGIAdapter2)
      || riid == __uuidof(IDXGIAdapter3)
+     || riid == __uuidof(IDXGIAdapter4)
      || riid == __uuidof(IDXGIDXVKAdapter)) {
       *ppvObject = ref(this);
       return S_OK;
@@ -135,14 +143,18 @@ namespace dxvk {
     if (ppOutput == nullptr)
       return E_INVALIDARG;
     
-    if (Output > 0) {
-      *ppOutput = nullptr;
-      return DXGI_ERROR_NOT_FOUND;
-    }
+    MonitorEnumInfo info;
+    info.iMonitorId = Output;
+    info.oMonitor   = nullptr;
     
-    // TODO support multiple monitors
-    HMONITOR monitor = ::MonitorFromPoint({ 0, 0 }, MONITOR_DEFAULTTOPRIMARY);
-    *ppOutput = ref(new DxgiOutput(m_factory, this, monitor));
+    ::EnumDisplayMonitors(
+      nullptr, nullptr, &MonitorEnumProc,
+      reinterpret_cast<LPARAM>(&info));
+    
+    if (info.oMonitor == nullptr)
+      return DXGI_ERROR_NOT_FOUND;
+    
+    *ppOutput = ref(new DxgiOutput(m_factory, this, info.oMonitor));
     return S_OK;
   }
   
@@ -151,8 +163,8 @@ namespace dxvk {
     if (pDesc == nullptr)
       return E_INVALIDARG;
 
-    DXGI_ADAPTER_DESC2 desc;
-    HRESULT hr = GetDesc2(&desc);
+    DXGI_ADAPTER_DESC3 desc;
+    HRESULT hr = GetDesc3(&desc);
     
     if (SUCCEEDED(hr)) {
       std::memcpy(pDesc->Description, desc.Description, sizeof(pDesc->Description));
@@ -175,8 +187,8 @@ namespace dxvk {
     if (pDesc == nullptr)
       return E_INVALIDARG;
 
-    DXGI_ADAPTER_DESC2 desc;
-    HRESULT hr = GetDesc2(&desc);
+    DXGI_ADAPTER_DESC3 desc;
+    HRESULT hr = GetDesc3(&desc);
     
     if (SUCCEEDED(hr)) {
       std::memcpy(pDesc->Description, desc.Description, sizeof(pDesc->Description));
@@ -199,6 +211,34 @@ namespace dxvk {
   HRESULT STDMETHODCALLTYPE DxgiAdapter::GetDesc2(DXGI_ADAPTER_DESC2* pDesc) {
     if (pDesc == nullptr)
       return E_INVALIDARG;
+
+    DXGI_ADAPTER_DESC3 desc;
+    HRESULT hr = GetDesc3(&desc);
+    
+    if (SUCCEEDED(hr)) {
+      std::memcpy(pDesc->Description, desc.Description, sizeof(pDesc->Description));
+      
+      pDesc->VendorId               = desc.VendorId;
+      pDesc->DeviceId               = desc.DeviceId;
+      pDesc->SubSysId               = desc.SubSysId;
+      pDesc->Revision               = desc.Revision;
+      pDesc->DedicatedVideoMemory   = desc.DedicatedVideoMemory;
+      pDesc->DedicatedSystemMemory  = desc.DedicatedSystemMemory;
+      pDesc->SharedSystemMemory     = desc.SharedSystemMemory;
+      pDesc->AdapterLuid            = desc.AdapterLuid;
+      pDesc->Flags                  = desc.Flags;
+      pDesc->GraphicsPreemptionGranularity = desc.GraphicsPreemptionGranularity;
+      pDesc->ComputePreemptionGranularity  = desc.ComputePreemptionGranularity;
+    }
+    
+    return hr;
+  }
+  
+  
+  HRESULT STDMETHODCALLTYPE DxgiAdapter::GetDesc3(
+          DXGI_ADAPTER_DESC3*       pDesc) {
+    if (pDesc == nullptr)
+      return E_INVALIDARG;
     
     const DxgiOptions* options = m_factory->GetOptions();
     
@@ -264,7 +304,7 @@ namespace dxvk {
     pDesc->DedicatedSystemMemory          = 0;
     pDesc->SharedSystemMemory             = sharedMemory;
     pDesc->AdapterLuid                    = LUID { 0, 0 };
-    pDesc->Flags                          = 0;
+    pDesc->Flags                          = DXGI_ADAPTER_FLAG3_NONE;
     pDesc->GraphicsPreemptionGranularity  = DXGI_GRAPHICS_PREEMPTION_DMA_BUFFER_BOUNDARY;
     pDesc->ComputePreemptionGranularity   = DXGI_COMPUTE_PREEMPTION_DMA_BUFFER_BOUNDARY;
 
@@ -275,8 +315,8 @@ namespace dxvk {
 
     return S_OK;
   }
-  
-  
+
+
   HRESULT STDMETHODCALLTYPE DxgiAdapter::QueryVideoMemoryInfo(
           UINT                          NodeIndex,
           DXGI_MEMORY_SEGMENT_GROUP     MemorySegmentGroup,
@@ -349,8 +389,23 @@ namespace dxvk {
   HRESULT STDMETHODCALLTYPE DxgiAdapter::RegisterVideoMemoryBudgetChangeNotificationEvent(
           HANDLE                        hEvent,
           DWORD*                        pdwCookie) {
-    Logger::err("DxgiAdapter::RegisterVideoMemoryBudgetChangeNotificationEvent: Not implemented");
-    return E_NOTIMPL;
+    if (!hEvent || !pdwCookie)
+      return E_INVALIDARG;
+
+    std::unique_lock<std::mutex> lock(m_mutex);
+    DWORD cookie = ++m_eventCookie;
+
+    m_eventMap.insert({ cookie, hEvent });
+
+    if (!m_eventThread.joinable())
+      m_eventThread = dxvk::thread([this] { runEventThread(); });
+
+    // This method seems to fire the
+    // event immediately on Windows
+    SetEvent(hEvent);
+
+    *pdwCookie = cookie;
+    return S_OK;
   }
   
 
@@ -362,7 +417,8 @@ namespace dxvk {
 
   void STDMETHODCALLTYPE DxgiAdapter::UnregisterVideoMemoryBudgetChangeNotification(
           DWORD                         dwCookie) {
-    Logger::err("DxgiAdapter::UnregisterVideoMemoryBudgetChangeNotification: Not implemented");
+    std::unique_lock<std::mutex> lock(m_mutex);
+    m_eventMap.erase(dwCookie);
   }
 
 
@@ -374,5 +430,51 @@ namespace dxvk {
   Rc<DxvkInstance> STDMETHODCALLTYPE DxgiAdapter::GetDXVKInstance() {
     return m_factory->GetDXVKInstance();
   }
+
+
+  void DxgiAdapter::runEventThread() {
+    env::setThreadName(str::format("dxvk-adapter-", m_index));
+
+    std::unique_lock<std::mutex> lock(m_mutex);
+    DxvkAdapterMemoryInfo memoryInfoOld = m_adapter->getMemoryHeapInfo();
+
+    while (true) {
+      m_cond.wait_for(lock, std::chrono::milliseconds(1500),
+        [this] { return m_eventCookie == ~0u; });
+
+      if (m_eventCookie == ~0u)
+        return;
+
+      auto memoryInfoNew = m_adapter->getMemoryHeapInfo();
+      bool budgetChanged = false;
+
+      for (uint32_t i = 0; i < memoryInfoNew.heapCount; i++) {
+        budgetChanged |= memoryInfoNew.heaps[i].memoryBudget
+                      != memoryInfoOld.heaps[i].memoryBudget;
+      }
+
+      if (budgetChanged) {
+        memoryInfoOld = memoryInfoNew;
+
+        for (const auto& pair : m_eventMap)
+          SetEvent(pair.second);
+      }
+    }
+  }
+  
+  
+  BOOL CALLBACK DxgiAdapter::MonitorEnumProc(
+          HMONITOR                  hmon,
+          HDC                       hdc,
+          LPRECT                    rect,
+          LPARAM                    lp) {
+    auto data = reinterpret_cast<MonitorEnumInfo*>(lp);
+    
+    if (data->iMonitorId--)
+      return TRUE; /* continue */
+    
+    data->oMonitor = hmon;
+    return FALSE; /* stop */
+  }
   
 }
