@@ -31,7 +31,9 @@ namespace dxvk {
      || riid == __uuidof(IDXGIFactory2)
      || riid == __uuidof(IDXGIFactory3)
      || riid == __uuidof(IDXGIFactory4)
-     || riid == __uuidof(IDXGIFactory5)) {
+     || riid == __uuidof(IDXGIFactory5)
+     || riid == __uuidof(IDXGIFactory6)
+     || riid == __uuidof(IDXGIFactory7)) {
       *ppvObject = ref(this);
       return S_OK;
     }
@@ -223,6 +225,33 @@ namespace dxvk {
   }
 
   
+  HRESULT STDMETHODCALLTYPE DxgiFactory::EnumAdapterByGpuPreference(
+          UINT                  Adapter,
+          DXGI_GPU_PREFERENCE   GpuPreference,
+          REFIID                riid,
+          void**                ppvAdapter) {
+    InitReturnPtr(ppvAdapter);
+    uint32_t adapterCount = m_instance->adapterCount();
+
+    if (Adapter >= adapterCount)
+      return DXGI_ERROR_NOT_FOUND;
+
+    // We know that the backend lists dedicated GPUs before
+    // any integrated ones, so just list adapters in reverse
+    // order. We have no other way to estimate performance.
+    if (GpuPreference == DXGI_GPU_PREFERENCE_MINIMUM_POWER)
+      Adapter = adapterCount - Adapter - 1;
+
+    Com<IDXGIAdapter> adapter;
+    HRESULT hr = this->EnumAdapters(Adapter, &adapter);
+
+    if (FAILED(hr))
+      return hr;
+
+    return adapter->QueryInterface(riid, ppvAdapter);
+  }
+
+
   HRESULT STDMETHODCALLTYPE DxgiFactory::EnumWarpAdapter(
           REFIID                riid,
           void**                ppvAdapter) {
@@ -343,4 +372,20 @@ namespace dxvk {
     }
   }
 
+
+  HRESULT STDMETHODCALLTYPE DxgiFactory::RegisterAdaptersChangedEvent(
+          HANDLE                hEvent,
+          DWORD*                pdwCookie) {
+    Logger::err("DxgiFactory: RegisterAdaptersChangedEvent: Stub");
+    return E_NOTIMPL;
+  }
+
+
+  HRESULT STDMETHODCALLTYPE DxgiFactory::UnregisterAdaptersChangedEvent(
+          DWORD                 Cookie) {
+    Logger::err("DxgiFactory: UnregisterAdaptersChangedEvent: Stub");
+    return E_NOTIMPL;
+  }
+
+
 }
