@@ -263,7 +263,7 @@ namespace dxvk {
       VK_FORMAT_R16_UINT },
     // DXGI_FORMAT_R16_FLOAT
     { VK_FORMAT_R16_SFLOAT,
-      VK_FORMAT_UNDEFINED,
+      VK_FORMAT_D16_UNORM,
       VK_FORMAT_R16_UINT,
       VK_IMAGE_ASPECT_COLOR_BIT },
     // DXGI_FORMAT_D16_UNORM
@@ -518,12 +518,10 @@ namespace dxvk {
     // DXGI_FORMAT_A8P8
     { }, // Unsupported
     // DXGI_FORMAT_B4G4R4A4_UNORM
-    { VK_FORMAT_B4G4R4A4_UNORM_PACK16,
+    { VK_FORMAT_A4R4G4B4_UNORM_PACK16_EXT,
       VK_FORMAT_UNDEFINED,
       VK_FORMAT_UNDEFINED,
-      VK_IMAGE_ASPECT_COLOR_BIT, 0,
-      { VK_COMPONENT_SWIZZLE_G, VK_COMPONENT_SWIZZLE_R,
-        VK_COMPONENT_SWIZZLE_A, VK_COMPONENT_SWIZZLE_B }},
+      VK_IMAGE_ASPECT_COLOR_BIT },
     // DXGI_FORMAT_P208
     { }, // Unsupported
     // DXGI_FORMAT_V208
@@ -842,6 +840,12 @@ namespace dxvk {
       RemapDepthFormat(DXGI_FORMAT_X24_TYPELESS_G8_UINT,  VK_FORMAT_D32_SFLOAT_S8_UINT);
       RemapDepthFormat(DXGI_FORMAT_D24_UNORM_S8_UINT,     VK_FORMAT_D32_SFLOAT_S8_UINT);
     }
+
+    if (!adapter->features().ext4444Formats.formatA4R4G4B4) {
+      RemapColorFormat(DXGI_FORMAT_B4G4R4A4_UNORM, VK_FORMAT_B4G4R4A4_UNORM_PACK16,
+        { VK_COMPONENT_SWIZZLE_G, VK_COMPONENT_SWIZZLE_R,
+          VK_COMPONENT_SWIZZLE_A, VK_COMPONENT_SWIZZLE_B });
+    }
   }
   
   
@@ -853,26 +857,16 @@ namespace dxvk {
   DXGI_VK_FORMAT_INFO DXGIVkFormatTable::GetFormatInfo(
           DXGI_FORMAT         Format,
           DXGI_VK_FORMAT_MODE Mode) const {
-    const DXGI_VK_FORMAT_MAPPING* mapping = GetFormatMapping(Format);
-    
-    switch (Mode) {
-      case DXGI_VK_FORMAT_MODE_ANY:
-        return mapping->FormatColor != VK_FORMAT_UNDEFINED
-          ? DXGI_VK_FORMAT_INFO { mapping->FormatColor, mapping->AspectColor, mapping->Swizzle }
-          : DXGI_VK_FORMAT_INFO { mapping->FormatDepth, mapping->AspectDepth };
-      
-      case DXGI_VK_FORMAT_MODE_COLOR:
-        return { mapping->FormatColor, mapping->AspectColor, mapping->Swizzle };
-      
-      case DXGI_VK_FORMAT_MODE_DEPTH:
-        return { mapping->FormatDepth, mapping->AspectDepth };
-      
-      case DXGI_VK_FORMAT_MODE_RAW:
-        return { mapping->FormatRaw, mapping->AspectColor };
-    }
-    
-    Logger::err("DXGI: GetFormatInfo: Internal error");
-    return DXGI_VK_FORMAT_INFO();
+    return GetFormatInfoFromMapping(
+      GetFormatMapping(Format), Mode);
+  }
+
+
+  DXGI_VK_FORMAT_INFO DXGIVkFormatTable::GetPackedFormatInfo(
+          DXGI_FORMAT         Format,
+          DXGI_VK_FORMAT_MODE Mode) const {
+    return GetFormatInfoFromMapping(
+      GetPackedFormatMapping(Format), Mode);
   }
   
   
@@ -890,6 +884,30 @@ namespace dxvk {
   }
 
 
+  DXGI_VK_FORMAT_INFO DXGIVkFormatTable::GetFormatInfoFromMapping(
+    const DXGI_VK_FORMAT_MAPPING* pMapping,
+          DXGI_VK_FORMAT_MODE   Mode) const {
+    switch (Mode) {
+      case DXGI_VK_FORMAT_MODE_ANY:
+        return pMapping->FormatColor != VK_FORMAT_UNDEFINED
+          ? DXGI_VK_FORMAT_INFO { pMapping->FormatColor, pMapping->AspectColor, pMapping->Swizzle }
+          : DXGI_VK_FORMAT_INFO { pMapping->FormatDepth, pMapping->AspectDepth };
+      
+      case DXGI_VK_FORMAT_MODE_COLOR:
+        return { pMapping->FormatColor, pMapping->AspectColor, pMapping->Swizzle };
+      
+      case DXGI_VK_FORMAT_MODE_DEPTH:
+        return { pMapping->FormatDepth, pMapping->AspectDepth };
+      
+      case DXGI_VK_FORMAT_MODE_RAW:
+        return { pMapping->FormatRaw, pMapping->AspectColor };
+    }
+    
+    Logger::err("DXGI: GetFormatInfoFromMapping: Internal error");
+    return DXGI_VK_FORMAT_INFO();
+  }
+
+
   const DXGI_VK_FORMAT_MAPPING* DXGIVkFormatTable::GetFormatMapping(
           DXGI_FORMAT         Format) const {
     const size_t formatId = size_t(Format);
@@ -900,6 +918,16 @@ namespace dxvk {
   }
   
 
+  const DXGI_VK_FORMAT_MAPPING* DXGIVkFormatTable::GetPackedFormatMapping(
+          DXGI_FORMAT         Format) const {
+    const size_t formatId = size_t(Format);
+    
+    return formatId < g_dxgiFormats.size()
+      ? &g_dxgiFormats[formatId]
+      : &g_dxgiFormats[0];
+  }
+  
+
   bool DXGIVkFormatTable::CheckImageFormatSupport(
     const Rc<DxvkAdapter>&      Adapter,
           VkFormat              Format,
@@ -917,4 +945,13 @@ namespace dxvk {
     m_dxgiFormats[uint32_t(Format)].FormatDepth = Target;
   }
   
+
+  void DXGIVkFormatTable::RemapColorFormat(
+          DXGI_FORMAT         Format,
+          VkFormat            Target,
+          VkComponentMapping  Swizzle) {
+    m_dxgiFormats[uint32_t(Format)].FormatColor = Target;
+    m_dxgiFormats[uint32_t(Format)].Swizzle = Swizzle;
+  }
+  
 }
\ No newline at end of file
