@@ -42,6 +42,7 @@ namespace dxvk {
    */
   struct DxvkDevicePerfHints {
     VkBool32 preferFbDepthStencilCopy : 1;
+    VkBool32 preferFbResolve          : 1;
   };
   
   /**
@@ -303,6 +304,17 @@ namespace dxvk {
     Rc<DxvkImage> createImage(
       const DxvkImageCreateInfo&  createInfo,
             VkMemoryPropertyFlags memoryType);
+
+    /**
+     * \brief Creates an image object for an existing VkImage
+     * 
+     * \param [in] createInfo Image create info
+     * \param [in] image Vulkan image to wrap
+     * \returns The image object
+     */
+    Rc<DxvkImage> createImageFromVkImage(
+      const DxvkImageCreateInfo&  createInfo,
+            VkImage               image);
     
     /**
      * \brief Creates an image view
