@@ -2,6 +2,7 @@
 
 #include "dxvk_instance.h"
 #include "dxvk_openvr.h"
+#include "dxvk_openxr.h"
 #include "dxvk_platform_exts.h"
 
 #include <algorithm>
@@ -23,6 +24,9 @@ namespace dxvk {
     if (m_options.enableOpenVR)
       m_extProviders.push_back(&VrInstance::s_instance);
 
+    if (m_options.enableOpenXR)
+      m_extProviders.push_back(&DxvkXrProvider::s_instance);
+
     Logger::info("Built-in extension providers:");
     for (const auto& provider : m_extProviders)
       Logger::info(str::format("  ", provider->getName()));
@@ -118,7 +122,7 @@ namespace dxvk {
     appInfo.pApplicationName      = appName.c_str();
     appInfo.applicationVersion    = 0;
     appInfo.pEngineName           = "DXVK";
-    appInfo.engineVersion         = VK_MAKE_VERSION(1, 5, 4);
+    appInfo.engineVersion         = VK_MAKE_VERSION(1, 8, 1);
     appInfo.apiVersion            = VK_MAKE_VERSION(1, 1, 0);
     
     VkInstanceCreateInfo info;
@@ -165,8 +169,21 @@ namespace dxvk {
     
     std::sort(result.begin(), result.end(),
       [] (const Rc<DxvkAdapter>& a, const Rc<DxvkAdapter>& b) -> bool {
-        return a->deviceProperties().deviceType == VK_PHYSICAL_DEVICE_TYPE_DISCRETE_GPU
-            && b->deviceProperties().deviceType != VK_PHYSICAL_DEVICE_TYPE_DISCRETE_GPU;
+        static const std::array<VkPhysicalDeviceType, 3> deviceTypes = {{
+          VK_PHYSICAL_DEVICE_TYPE_DISCRETE_GPU,
+          VK_PHYSICAL_DEVICE_TYPE_INTEGRATED_GPU,
+          VK_PHYSICAL_DEVICE_TYPE_VIRTUAL_GPU,
+        }};
+
+        uint32_t aRank = deviceTypes.size();
+        uint32_t bRank = deviceTypes.size();
+
+        for (uint32_t i = 0; i < std::min(aRank, bRank); i++) {
+          if (a->deviceProperties().deviceType == deviceTypes[i]) aRank = i;
+          if (b->deviceProperties().deviceType == deviceTypes[i]) bRank = i;
+        }
+
+        return aRank < bRank;
       });
     
     if (result.size() == 0) {
