@@ -44,6 +44,7 @@ namespace dxvk {
   struct DxvkMemoryHeap {
     VkMemoryHeap      properties;
     DxvkMemoryStats   stats;
+    VkDeviceSize      budget;
   };
 
 
@@ -285,8 +286,6 @@ namespace dxvk {
     std::array<DxvkMemoryHeap, VK_MAX_MEMORY_HEAPS> m_memHeaps;
     std::array<DxvkMemoryType, VK_MAX_MEMORY_TYPES> m_memTypes;
 
-    bool m_restrictAllocations;
-    
     DxvkMemory tryAlloc(
       const VkMemoryRequirements*             req,
       const VkMemoryDedicatedAllocateInfo*    dedAllocInfo,
