@@ -14,6 +14,9 @@ namespace dxvk {
     /// Enables OpenVR loading
     bool enableOpenVR;
 
+    /// Enables OpenXR loading
+    bool enableOpenXR;
+
     /// Number of compiler threads
     /// when using the state cache
     int32_t numCompilerThreads;
