@@ -176,7 +176,7 @@ namespace dxvk {
 
     void writerFunc();
 
-    std::string getCacheFileName() const;
+    std::wstring getCacheFileName() const;
     
     std::string getCacheDir() const;
 
@@ -191,4 +191,4 @@ namespace dxvk {
 
   };
 
-}
\ No newline at end of file
+}
